"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const constants_1 = require("./constants");
const cache = {};
class NestRPC {
    static register(ctx) {
        electron_1.ipcMain.on(constants_1.NEST_RPC_INVOKE_EVENT, (e, cls, method, evt, ...params) => __awaiter(this, void 0, void 0, function* () {
            const instance = cache[cls] || ctx.get(cls);
            if (!cache[cls]) {
                cache[cls] = instance;
            }
            try {
                params = NestRPC.handleParams(params, e, evt.callbacks);
                const data = yield instance[method].apply(instance, ...params);
                e.sender.send(evt.event, null, data);
            }
            catch (err) {
                e.sender.send(evt.event, err.message || 'Unknown Exception', null);
            }
        }));
    }
    static handleParams(params, e, callbacks) {
        const parameters = params.map((param, index) => {
            if (param.type === 'function') {
                const evt = callbacks.filter(cb => cb.index === index)[0];
                if (evt) {
                    return (...data) => e.sender.send(evt.event, ...data);
                }
                else {
                    return (...data) => void 0;
                }
            }
            else if (param.type === 'object' || param.type === 'boolean') {
                return JSON.parse(param.value);
            }
            else {
                return param.value;
            }
        });
        return parameters;
    }
}
exports.NestRPC = NestRPC;
