"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const uuid = require("uuid/v1");
const constants_1 = require("./constants");
const RPCException_1 = require("./RPCException");
exports.nestRPC = (cls) => {
    for (const key in cls.prototype) {
        const method = key;
        Object.defineProperty(cls.prototype, method, {
            value: (...params) => __awaiter(this, void 0, void 0, function* () {
                const result = handleParams(cls, method, params);
                const event = `${constants_1.NEST_RPC_INVOKE_RESPONSE_EVENT}__${cls.name}__${method}__${uuid()}`;
                result.callbackEvents.forEach(item => {
                    electron_1.ipcRenderer.once(item.event, (e, ...data) => {
                        const callback = params[item.index];
                        if (typeof callback === 'function') {
                            callback.apply(cls.prototype, ...data);
                        }
                    });
                });
                electron_1.ipcRenderer.send(constants_1.NEST_RPC_INVOKE_EVENT, cls.name, method, {
                    event,
                    callbacks: result.callbackEvents
                }, ...result.parameters);
                return new Promise((resolve, reject) => {
                    electron_1.ipcRenderer.once(event, (e, msg, ...params) => {
                        msg ? reject(new RPCException_1.RPCException(msg)) : resolve(...params);
                    });
                });
            })
        });
    }
    return cls.prototype;
};
function handleParams(cls, method, params) {
    const callbackEvents = [];
    const parameters = params.map((param, index) => {
        if (typeof param === 'function') {
            const event = `${constants_1.NEST_RPC_CALLBACK_EVENT}__${cls.name}__${method}__${index}__${uuid()}`;
            callbackEvents.push({ event, index });
            return { type: 'function', value: event };
        }
        else if (typeof param === 'object' || typeof param === 'boolean') {
            return { type: typeof param, value: JSON.stringify(param) };
        }
        else {
            return { type: typeof param, value: param };
        }
    });
    return { callbackEvents, parameters };
}
